#include "EtaSegRotatedMix.h"



EtaSegRotatedMix::EtaSegRotatedMix(){
	hnTotGood = new TH1D("nTotGood","nTotGood", 150, 0., 150.) ;
	deltaeta = new TH1D("deltaeta","deltaeta", 200, -7, 7) ;
	deltaphi = new TH1D("deltaphi","deltaphi", 200, -7, 7) ;
	deltaNN = new TH1D("deltann","deltann", 100, -20, 20) ;
	double delta = (rangemax - rangemin)/(double)(maxnbins);
	etabinned= new TH1D("etabinnedpost","etabinnedpost",maxnbins,rangemin,rangemax);
	etabinnedp= new TH1D("etabinnedpre","etabinnedpre",maxnbins,rangemin,rangemax);
	for (int k =0; k< maxnbins; k++){
		rangehist.push_back(new TH1D(Form("range%3.1fto%3.1f",(rangemin+k*delta),(rangemin+(k+1)*delta)),Form("range%3.1fto%3.1f",(rangemin+k*delta),(rangemin+(k+1)*delta)), 100, 0, 20));
	}
	matrixlist = new vector<matrixcontainer>; // Indice della classe
	indexed = false; // Parametro di controllo dell'indice
	rotationmatrix = new TRotation(); // Matrice di rotazione
	lastid = -1; // Id di partenza
	
}


Double_t EtaSegRotatedMix::ComputeEventWeight() // compute eta of event
{
	double delta = (rangemax - rangemin)/(double)(maxnbins);
	bool findin = false;
	vector<binsort> counterta;
	int counted=0;
	binsort a;
	a.weigth =0;
	for (int k =0; k< maxnbins; k++){
		a.binnumber = k;
		counterta.push_back(a);
	}
	for( unsigned int i=0 ; i< GoodTrackFourVector->size() ; i++ ) 
    {
		Double_t eta = GoodTrackFourVector->at(i).Eta() ;
		etabinnedp->Fill(eta);
		findin = false;
		for (int k =0; k< maxnbins; k++){
			if ( ((rangemin+k*delta) < eta) &&  (eta <=(rangemin+(k+1)*delta))){
				counterta[k].weigth++;
				counted++;
				
			}
		}
    }
	for (int k=0; k< maxnbins; k++){
		rangehist[k]->Fill((Double_t)counterta[k].weigth);
		etabinned->Fill((rangemin+(k+0.5)*delta),(Double_t)counterta[k].weigth);
		// etabinned->SetBinContent(k+1,(Double_t)counterta[k].weigth+etabinned->GetBinContent(k+1));
	}
	if ((GoodTrackFourVector->size()-counted)!=0) cout << "Warning, some trace not in range" << endl;
	hnTotGood->Fill(GoodTrackFourVector->size()) ;
	Double_t ReturnValue = pow(10,10)*GoodTrackFourVector->size()+Encode(counterta);
	
	
	Double_t SumP2 =0 ; 
	Double_t SphT[]={0,0,0,0,0,0,0,0,0} ; // Inizializzazione di un vettore vuoto per la matrice di sfericita'
	matrixcontainer b;
	//for (int k=0;k<9;k++) a.matrix[k] =0; // Azzero il vettore che uso per salvare gli autovettori
	
	// Calcolo delle entrate matrice di sfericita
	for( unsigned int i=0 ; i< GoodTrackFourVector->size() ; i++ )
    {
		SphT[0] += GoodTrackFourVector->at(i).Px()*GoodTrackFourVector->at(i).Px() ;
		SphT[1] += GoodTrackFourVector->at(i).Px()*GoodTrackFourVector->at(i).Py() ;
		SphT[2] += GoodTrackFourVector->at(i).Px()*GoodTrackFourVector->at(i).Pz() ;
		
		SphT[3] += GoodTrackFourVector->at(i).Py()*GoodTrackFourVector->at(i).Px() ;
		SphT[4] += GoodTrackFourVector->at(i).Py()*GoodTrackFourVector->at(i).Py() ; 
		SphT[5] += GoodTrackFourVector->at(i).Py()*GoodTrackFourVector->at(i).Pz() ; 
		
		SphT[6] += GoodTrackFourVector->at(i).Pz()*GoodTrackFourVector->at(i).Px() ;
		SphT[7] += GoodTrackFourVector->at(i).Pz()*GoodTrackFourVector->at(i).Py() ; 
		SphT[8] += GoodTrackFourVector->at(i).Pz()*GoodTrackFourVector->at(i).Pz() ;
		
		SumP2 += GoodTrackFourVector->at(i).P() * GoodTrackFourVector->at(i).P() ;
    }
	
	if( SumP2 == 0 ) { cout << "ERROR ERROR ERRROR" << endl; return -1 ;}
	
	TMatrixDSym SphM( 3, SphT ) ; // Creazione della matrice
	
	SphM *= 1./SumP2 ;
	TMatrixDSymEigen SphE( SphM ) ;
	b.thematrix.ResizeTo(3,3);
	b.thematrix = SphE.GetEigenVectors();
	b.weigth = ReturnValue;
	matrixlist->push_back(b); // Inserisco nell'indice di questa classe
	return ReturnValue ;
}

Double_t EtaSegRotatedMix::Encode( vector<binsort> & caselle )
{
	Double_t coded = 0;
	sort(caselle.begin(),caselle.end());
	for (unsigned int mm=0; mm<caselle.size(); mm++) {
		coded += caselle[mm].binnumber*pow(10,maxnbins - mm);
	}
	return coded;
}

EtaSegRotatedMix::~EtaSegRotatedMix(){
	hnTotGood->Write();
	deltaeta->Write();
	deltaphi->Write();
	for (int k =0; k< maxnbins; k++){
		rangehist[k]->Write();
		delete rangehist[k];
	}
	deltaNN->Write();
	etabinned->Write();
	etabinnedp->Write();
	delete deltaNN;
	delete deltaeta;
	delete deltaphi;
	delete hnTotGood;
	delete etabinnedp;
	delete etabinned;
	delete matrixlist;
	delete rotationmatrix;
}


TMatrixD EtaSegRotatedMix::GetMatrixFromId(int evid){
	if (!indexed){ indexed=true; sort(matrixlist->begin(),matrixlist->end());} // Nel caso nn sia ordinato la ordino
	return matrixlist->at(evid).thematrix; // ritorno l'array
}

// ======================================================================
// 
// ======================================================================
void EtaSegRotatedMix::multianalisis( const TLorentzVector &p1, const TLorentzVector &p2  , int &ChargeP1,int &ChargeP2, int SizeEv1 ,int SizeEv2 ,int evid){
	TLorentzVector newvector = p2; // Vettore di supporto
	if (evid!=lastid) {
		TMatrixD rota =GetMatrixFromId(evid);
		TMatrixD rotb =GetMatrixFromId(evid+1);
		//TMatrixD rota =GetMatrixFromId(evid)*GetMatrixFromId(evid+1);
		//rota*=rota.Determinant() ;
		// preparo i nuovi assi come differenza degli autovettori di sfericta dei due eventi
		TVector3 sferv1 (rota[0][0],rota[0][1],rota[0][2]);
		TVector3 sferv2 (rotb[0][0],rotb[0][1],rotb[0][2]);
		Double_t angle = -sferv1.Phi()+sferv2.Phi();
		TVector3 newX (cos(angle),-sin(angle),0);
		TVector3 newY (sin(angle),cos(angle),0);
		TVector3 newZ (0,0,1);
		delete rotationmatrix;
		rotationmatrix = new TRotation();
		rotationmatrix->RotateAxes(newX,newY,newZ); // Creo la matrice di rotazione.
		TVector3 sfervt = sferv2;
		sfervt.Transform(*rotationmatrix);
		//cout << "\n" << evid << endl;
		//cout << sferv1.Phi() << " " << sferv2.Phi() << " " << sfervt.Phi() << endl;
		//cout <<  sferv1.Phi()-sferv2.Phi()  << ":" << sferv1.Phi()-sfervt.Phi() << "\n" << endl;
		
		lastid=evid;
	}
	newvector.Transform(*rotationmatrix); // Ruoto la traccia
	
	Double_t q = GetQ(p1,newvector); // Calcolo la correlazione
	
	hmix->Fill(q) ; // riempio l'istogramma
	if (ChargeP1*ChargeP2 > 0){ 
		hmixsc->Fill(q) ;
		hmixschVsN->Fill(q,SizeEv1);
	}else{
		hmixoc->Fill(q) ;
		hmixdchVsN->Fill(q,SizeEv1);
	}
	
	
	deltaeta->Fill(p1.Eta()-p2.Eta());
	deltaphi->Fill(p1.Phi()-p2.Phi());
	deltaNN->Fill(SizeEv1-SizeEv2);
};


