#ifndef EventMixer_H
#define EventMixer_H

#include "TreeReindex.h"
#include <TLorentzVector.h>
#include <TMatrixD.h>
#include "remapper.c"
#include <TH1D.h>
#include <TH2D.h>
#include <TH3D.h>
#include <fstream>
#include <TFile.h>

class EventMixer: public TreeReindex {
	
public:
	
    EventMixer();
    ~EventMixer();
	
	int combination;	
    virtual void analyze( int entry );
    virtual void begin_analysis();
    virtual void end_analysis();
	virtual Double_t ComputeEventWeight()=0;
	virtual Double_t ComputeEventSecWeight(){return 0;};
	virtual Double_t ComputeEventTrdWeight(){return 0;};
	virtual void SaveToFile( char * );
    virtual void EvtSort(int maxentry=0) ;
	virtual void Loop( int evtmax = 9999999, int ifirst = 0 );
	Double_t GetQ( const TLorentzVector &p1, const TLorentzVector &p2 );
	Double_t GetQlong( const TLorentzVector &p1, const TLorentzVector &p2 );
	Double_t GetQout( const TLorentzVector &p1, const TLorentzVector &p2 );
	Double_t GetQside( const TLorentzVector &p1, const TLorentzVector &p2 );
    const Double_t CoulombWpm( const Double_t& q);
	const Double_t CoulombW( const Double_t& q);
	const TLorentzVector InvertPVector( TLorentzVector &vec);
	const TLorentzVector InvertXYVector( TLorentzVector &vec);
	bool splitcomb(TLorentzVector &vec1,TLorentzVector &vec2);
	void SignalCalculator(vector<TLorentzVector> & GTVector,vector<int> &GTCharge);
	void DisableSignalCalculator();
	void DisableMixCalculator();
	void EnableSignalCalculator();
	void EnableMixCalculator();
	void singleanalisis( const TLorentzVector &p1, const TLorentzVector &p2, int &ChargeP1,int &ChargeP2 ){;};
	virtual void multianalisis( const TLorentzVector &p1, const TLorentzVector &p2  , int &ChargeP1,int &ChargeP2, int SizeEv1 ,int SizeEv2 ,int evid) =0;
	void defaultmultianalisis( const TLorentzVector &p1, const TLorentzVector &p2  , int &ChargeP1,int &ChargeP2, int SizeEv1 ,int SizeEv2 ,int evid);
	
	TFile *outgraph;
private:
    bool MC;
	
    EventMixer( const TreeReindex& st );
    EventMixer& operator=( const TreeReindex& st );
	static const Double_t pimass = 0.1396;
	static const Double_t kamass = 0.4937 ; 
	static const Double_t cos_cut = 0.99996;
	static const Double_t dpt_cut = 0.04 ;
	bool signalcalc;
	bool mixcalc;
	
protected:	
	// Grafici
	TH1D* hmix;
	TH1D* hmixsc;
	TH1D* hmixoc;
	TH1D* hChargeP;
	TH2D* hEtaVsPtGoodP;
	TH1D* hPhiP;
	TH1D* hChargeM;
	TH2D* hEtaVsPtGoodM;
	TH1D* hPhiM;
	TH1D* hqschnc;
	TH1D* hqsch;
	TH1D* hqosch;
	TH1D* hqotsch;
	TH1D* hqdchnc;
	TH1D* hqdch;
	TH1D* hqodch;
	TH1D* hqotdch;
	TH1D* hN1MinusN2;
	TH2D* hN1VsN2;
	// multiplicity
	TH2D* hqschncVsN;
	TH2D* hqschVsN;
	TH2D* hqoschVsN;
	TH2D* hqotschVsN;
	TH2D* hqdchncVsN;
	TH2D* hqdchVsN;
	TH2D* hqodchVsN;
	TH2D* hqotdchVsN;
	TH2D* hmixschVsN;
	TH2D* hmixdchVsN;
	
	// PT
	
	TH2D* hqschncVsPt;
	TH2D* hqschVsPt;
	TH2D* hqoschVsPt;
	TH2D* hqotschVsPt;
	TH2D* hqdchncVsPt;
	TH2D* hqdchVsPt;
	TH2D* hqodchVsPt;
	TH2D* hqotdchVsPt;
	
	TH2D* hmixschVsPt;
	TH2D* hmixdchVsPt;
  
    TH2D* hqschncVsPtSum ;
    TH2D* hqschVsPtSum ;
    TH2D* hmixschVsPtSum;
    
    TH3D* hschDphiDeta ;
    TH3D* hdchDphiDeta;
    TH3D* hmixschDphiDeta;
    TH3D* hmixdchDphiDeta;
  
	// 3D
	
	TH3D* hqschncVsKtVsNch;
	TH3D* hqschVsKtVsNch;
	TH3D* hqoschVsKtVsNch;
	TH3D* hqotschVsKtVsNch;
	TH3D* hqdchncVsKtVsNch;
	TH3D* hqdchVsKtVsNch;
	TH3D* hqodchVsKtVsNch;
	TH3D* hqotdchVsKtVsNch;
	
	TH3D* hmixschVsKtVsNch;
	TH3D* hmixdchVsKtVsNch;
};

#endif
