#include "TrackSelector.h"
#include <TH2D.h>
#include <TH2.h>
#include <TVector3.h>
#include <TLorentzVector.h>
#include "TMatrixD.h" 
#include "TMatrixDSym.h" 
#include "TMatrixDSymEigen.h" 
#include <iostream>
#include <vector>
#include <math.h>
#include <string>
#include <TFile.h>

using namespace std;

static const Long64_t offset = (Long64_t) pow(10,7) ;
static const int MaxEvt=1500000 ;

bool Used[MaxEvt] = { MaxEvt * 0 } ;

TrackSelector::TrackSelector() 
{
	GoodTrackFourVector = new vector <TLorentzVector> ;
	GoodTrackCharge = new vector <int>;
	countertotal =0;
	enable_graph = false;
	goodeventsfound = 0;
}

void TrackSelector::InitOutput(string filename){
	fileoutput = new TFile(filename.c_str(),"recreate");
	fileoutput->cd();
	OutputTree = new TTree("STracks","SelectedTracks");
	OutputTree->Branch("GoodTrackFourVector","vector <TLorentzVector>",&GoodTrackFourVector);
	OutputTree->Branch("GoodTrackCharge","vector <int>",&GoodTrackCharge);
	OutputTree->Branch("evNumber",&EventId,"evNumber/I");
	OutputTree->Branch("RunNumber",&runNumber,"RunNumber/I");
	OutputTree->Write();
	//cout << " Genp_idx_clst " << Genp_idx_clst->at(i) << endl;
	trk_charge = new TH1D("trk_charge","trk_charge",4,-1.5,1.5) ;
	trk_pt = new TH1D("trk_pt","trk_pt",200,0.,50.) ;
	trk_p= new TH1D("trk_p","trk_p",200,0.,50.);
	trk_eta= new TH1D("trk_eta","trk_eta",200,-3.,3.);
	trk_phi= new TH1D("trk_phi","trk_phi",200,-4.,4.);
	trk_dxy= new TH1D("trk_dxy","trk_dxy",200,0.,0.5);
	trk_dz= new TH1D("trk_dz","trk_dz",200,-20.,20.);
	trk_d0= new TH1D("trk_d0","trk_d0",200,0.,0.5);
	trk_chi2= new TH1D("trk_chi2","trk_chi2",200,0.,10.);
	trk_validHits= new TH1D("trk_validHits","trk_validHits",50,0.,50.);
	trk_LostHits= new TH1D("trk_LostHits","trk_LostHits",50,0.,50.);
	goodtrk_charge= new TH1D("goodtrk_charge","goodtrk_charge",4,-1.5,1.5);
	goodtrk_pt= new TH1D("goodtrk_pt","goodtrk_pt",200,0.,50.);
	goodtrk_p= new TH1D("goodtrk_p","goodtrk_p",200,0.,50.);
	goodtrk_eta= new TH1D("goodtrk_eta","goodtrk_eta",200,-3.,3.);
	goodtrk_phi= new TH1D("goodtrk_phi","goodtrk_phi",200,-4.,4.);
	goodtrk_dxy= new TH1D("goodtrk_dxy","goodtrk_dxy",200,0.,0.5);
	goodtrk_dz= new TH1D("goodtrk_dz","goodtrk_dz",200,-20.,20.);
	goodtrk_d0= new TH1D("goodtrk_d0","goodtrk_d0",200,0.,0.5);
	goodtrk_chi2= new TH1D("goodtrk_chi2","goodtrk_chi2",200,0.,10.);
	goodtrk_validHits= new TH1D("goodtrk_validHits","goodtrk_validHits",50,0.,50.);
	goodtrk_LostHits= new TH1D("goodtrk_LostHits","goodtrk_LostHits",50,0.,50.);
	GoodTracks = new TH1D ("GoodTracks","GoodTracks",150,0.,150.) ;
	AllTracks = new TH1D ("AllTracks","AllTracks",150,0.,150.) ;
	
	
}

void TrackSelector::EnableGraphs(){
	enable_graph = true;
}

TrackSelector::~TrackSelector() 
{
	if (fileoutput){
		
		// save all graph
		trk_charge->Write();
		trk_pt->Write();
		trk_p->Write();
		trk_eta->Write();
		trk_phi->Write();
		trk_dxy->Write();
		trk_dz->Write();
		trk_d0->Write();
		trk_chi2->Write();
		trk_validHits->Write();
		trk_LostHits->Write();
		goodtrk_charge->Write();
		goodtrk_pt->Write();
		goodtrk_p->Write();
		goodtrk_eta->Write();
		goodtrk_phi->Write();
		goodtrk_dxy->Write();
		goodtrk_dz->Write();
		goodtrk_d0->Write();
		goodtrk_chi2->Write();
		goodtrk_validHits->Write();
		goodtrk_LostHits->Write();
		GoodTracks->Write();
		AllTracks->Write();
		
		// Destroy all graph
		
		delete trk_charge;
		delete trk_pt;
		delete trk_p;
		delete trk_eta;
		delete trk_phi;
		delete trk_dxy;
		delete trk_dz;
		delete trk_d0;
		delete trk_chi2;
		delete trk_validHits;
		delete trk_LostHits;
		delete goodtrk_charge;
		delete goodtrk_pt;
		delete goodtrk_p;
		delete goodtrk_eta;
		delete goodtrk_phi;
		delete goodtrk_dxy;
		delete goodtrk_dz;
		delete goodtrk_d0;
		delete goodtrk_chi2;
		delete goodtrk_validHits;
		delete goodtrk_LostHits; 
		delete GoodTracks;
		
		// Saving tree
		OutputTree->SetName("SelectedTracks");
		OutputTree->Write();
		// Remove old copy
		fileoutput->Delete("STracks;1");
		fileoutput->Close();
		
		delete fileoutput;
	}
	//delete fileoutput;
	delete GoodTrackFourVector;
	delete GoodTrackCharge;
}
// =======================================================================
void TrackSelector::AnalyzeTracks() { 
	int goodTracks=0;
	for( int i = 0 ; i < numberOfTrack ; i++ ) {
		//cout << " Genp_idx_clst " << Genp_idx_clst->at(i) << endl;
		trk_charge->Fill(charge->at(i));
		trk_pt->Fill(pt->at(i));
		trk_p->Fill(p->at(i));
		trk_eta->Fill(eta->at(i));
		trk_phi->Fill(phi->at(i));
		trk_dxy->Fill(dxy_PV->at(i));
		trk_dz->Fill(dz->at(i));
		trk_d0->Fill(d0->at(i));
		trk_chi2->Fill(normalizedChi2->at(i));
		trk_validHits->Fill(numberOfvalidHits->at(i));
		trk_LostHits->Fill(highPurity_track->at(i));		
		if( !GoodTrack.at(i) ) continue ;
		goodTracks++;
		goodtrk_charge->Fill(charge->at(i));
		goodtrk_pt->Fill(pt->at(i));
		goodtrk_p->Fill(p->at(i));
		goodtrk_eta->Fill(eta->at(i));
		goodtrk_phi->Fill(phi->at(i));
		goodtrk_dxy->Fill(dxy_PV->at(i));
		goodtrk_dz->Fill(dz->at(i));
		goodtrk_d0->Fill(d0->at(i));
		goodtrk_chi2->Fill(normalizedChi2->at(i));
		goodtrk_validHits->Fill(numberOfvalidHits->at(i));
		goodtrk_LostHits->Fill(highPurity_track->at(i));
		
		
	}
	GoodTracks->Fill(goodTracks) ;
	AllTracks->Fill(numberOfTrack) ;
}
// =======================================================================
void TrackSelector::analyze( int entry , vector<int>* chv ,vector <TLorentzVector>* tkv )
{
	analyze( entry ) ;
	tkv = GoodTrackFourVector ;
	chv = GoodTrackCharge ;
	return; 
}
// =======================================================================
void TrackSelector::analyze( int entry ) {
	
	if( numberOfTrack > 150 || numberOfTrack < 2) return ;   // sanity cut 
	EventId= evNumber;
	SelectGoodTracks() ;  // as defined in IsGoodTrack
//	RemoveSplitTracks(); 
	FillGoodTracks();
	AnalyzeTracks();
	if (GoodTrackFourVector->size() < 2) return; // 0 track removal 
	counterevents++;
	goodeventsfound++;
	countertotal+=GoodTrackFourVector->size();
	coutcomb+=((GoodTrackFourVector->size()-1)*GoodTrackFourVector->size())/2.;
	OutputTree->Fill();
	return;
}

// =======================================================================
// Select good track
// =======================================================================
inline void TrackSelector::SelectGoodTracks()
{
	TrackFourVector.erase( TrackFourVector.begin(), TrackFourVector.end() ) ;
	GoodTrack.erase( GoodTrack.begin() , GoodTrack.end() ) ;
	TLorentzVector pTrack ;
	// cout << "Original n of track" << numberOfTrack << " n event " << EventId << endl;
	for( int i = 0; i < numberOfTrack ; i++ ) 
    {
		pTrack.SetXYZM( px->at(i), py->at(i), pz->at(i), pimass ) ;
		TrackFourVector.push_back( pTrack ) ;
		if( IsGoodTrack(i) ) 
		{ 
			GoodTrack.push_back(true);
		}
		else
		{ GoodTrack.push_back(false); 
			// cout << i << " bad" << endl;
		}
	} 
	return ;
}
// =======================================================================
// remove tracks having cos(theta)>cst_split_cut and 
// abs(delta p) < dpt_splt_cut  with another equal charge track 
// fill also some control histo
// =======================================================================
inline void  TrackSelector::RemoveSplitTracks() {

  SplitTrack.erase( SplitTrack.begin(), SplitTrack.end() ) ;
  for( int i = 0 ; i<numberOfTrack ; i++ ) { SplitTrack.push_back( false ) ;}
  
  for( int i = 0 ; i<numberOfTrack ; i++ ) {
    if( !GoodTrack.at(i) ) continue ;

    for( int j = i+1 ; j<numberOfTrack ; j++ ) {
      if( !GoodTrack.at(j) ) continue ;

      double dpt = TrackFourVector.at(i).Pt()-TrackFourVector.at(j).Pt() ;
      double cst = cos( TrackFourVector.at(i).Angle( TrackFourVector.at(j).Vect() ) ) ;    

      if( charge->at(i) != charge->at(j) ) {
        continue ;
      }

      if( cst >cst_split_cut && fabs(dpt)< dpt_split_cut ) { 
        // remove both tracks!
        GoodTrack[j] = false ; 
        SplitTrack[j] = true ;
        GoodTrack[i] = false ; 
        SplitTrack[i] = true ;
      }
    }
  }
  return ;
}

// =======================================================================
inline void  TrackSelector::FillGoodTracks() 
{  
	GoodTrackFourVector->erase( GoodTrackFourVector->begin(), GoodTrackFourVector->end() ) ;
	GoodTrackCharge->erase( GoodTrackCharge->begin(), GoodTrackCharge->end() ) ;
	for( unsigned int i = 0 ; i<TrackFourVector.size() ; i++ )
    {
		if( !GoodTrack.at(i) ) continue ;
		GoodTrackCharge->push_back( charge->at(i) ) ;
		GoodTrackFourVector->push_back( TrackFourVector.at(i) ) ;
	}
	
	// cout << "Good track for event " << EventId << " " << GoodTrackFourVector->size() << endl;
	return ;
}
// =======================================================================
inline int TrackSelector::IsGoodTrack( int i ) { 
	return (RejectTrack(i)==0 ? 1 : 0  ); 
}
// =======================================================================
inline int TrackSelector::RejectTrack( int i ) {
	int ReturnValue = 0;
	
	if( !highPurity_track->at(i) )  {        ReturnValue += (int) pow(2,0) ;
		// cout << " Purity track id" << i << "  " << highPurity_track->at(i) << endl;
	}
	if( pt->at(i) <= pt_cut )        {       ReturnValue += (int) pow(2,1) ;
		// cout << " Pt track id" << i << "  " << pt->at(i) <<  " pt_cut " <<  pt_cut << endl;
	}
	Double_t inner_r=sqrt(innerPosition_x->at(i)*innerPosition_x->at(i)+innerPosition_y->at(i)*innerPosition_y->at(i));
	if( fabs( dxy_PV->at(i)) >= dxy_cut || inner_r>= rmin_cut ) 
		ReturnValue += (int) pow(2,2) ;
	if( normalizedChi2->at(i) >= nchi2_cut) ReturnValue += (int) pow(2,3) ;
	if( ndof->at(i) <= ndof_cut )           ReturnValue += (int) pow(2,4) ;
	if( fabs( eta->at(i) ) >= eta_cut )     ReturnValue += (int) pow(2,5) ;
	
	FillCutControlHisto( i, ReturnValue ) ;
	
	return ReturnValue ;
}

// =======================================================================
inline void TrackSelector::FillCutControlHisto( int itrack, int rcut ) {
	char htitle[100] ;
	for( int jcut=0 ; jcut<nTrackCut ; jcut++ ) 
    { 
		if( (rcut&(int) pow(2,jcut) ) ) continue ;
		sprintf( htitle, "etacut%d", jcut ) ;
		if( charge->at(itrack) > 0 ) 
		{ strcat( htitle,"_plus" ) ; }
		else
		{ strcat( htitle,"_minus" ) ; }
    }
} 
// ============================================================
int TrackSelector::GetCounterTotal(){
	return countertotal;
}
// ============================================================
int TrackSelector::GetGoodEvent(){
	return counterevents;
}
// ============================================================
