#ifndef TrackSelector_H
#define TrackSelector_H

#include <vector>
#include <TFile.h>
#include "TreeRead.h"
#include <TLorentzVector.h>
#include "TMatrixD.h"
#include <TH1D.h>
#include <TList.h>

class TrackSelector: public TreeRead {
	
public:
	
    TrackSelector();
    ~TrackSelector();
	
    virtual void analyze( int entry );
    virtual void analyze( int, vector<int>*, vector<TLorentzVector>* );
    virtual void begin_analysis() {};
    virtual void end_analysis() {};
    virtual int RejectTrack( int itrack );
    virtual int IsGoodTrack( int itrack );
    virtual void FillGoodTracks() ;
    virtual void FillCutControlHisto( int i, int r ) ;
    virtual void RemoveSplitTracks() ;
    virtual void SelectGoodTracks() ;
	int GetCounterTotal();
    void InitOutput(string filename);
	int GetGoodEvent();
	void EnableGraphs();
	double coutcomb;    
private:
    bool MC;
	bool enable_graph;
	
    TrackSelector( const TreeRead& st );
    TrackSelector& operator=( const TreeRead& st );
	int countertotal;
	int counterevents;
	
	//
	void AnalyzeTracks();
	vector <bool> GoodTrack ; // list of good tracks
	vector <bool> SplitTrack ; // list of split tracks
	
	vector <TLorentzVector> TrackFourVector ; 
	vector <TLorentzVector>* GoodTrackFourVector ;
	vector <int>* GoodTrackCharge ;
	// track quality cuts
	static const int nTrackCut = 6 ;
	
	static const Int_t ndof_cut = 5 ;
	static const Int_t nchi2_cut = 5 ;
	//static const Double_t dxy_cut = 1. ;
	static const Double_t dxy_cut = .15 ;
	static const Double_t rmin_cut = 20. ;
	static const Double_t rgen_cut= 20. ;
	static const Double_t pt_cut = 0.2 ;
	static const Double_t eta_cut = 2.4 ;
	// split track cuts
	static const Double_t cst_split_cut = 0.99996 ;
	static const Double_t dpt_split_cut = 0.04 ;
	
	static const Double_t pimass = 0.1396, kamass = 0.4937 ; 
	TFile *fileoutput;
	TTree* OutputTree;
	int EventId;
	TH1D* trk_charge;
	TH1D* trk_pt;
	TH1D* trk_p;
	TH1D* trk_eta;
	TH1D* trk_phi;
	TH1D* trk_dxy;
	TH1D* trk_dz;
	TH1D* trk_d0;
	TH1D* trk_chi2;
	TH1D* trk_validHits;
	TH1D* trk_LostHits;
	TH1D* goodtrk_charge;
	TH1D* goodtrk_pt;
	TH1D* goodtrk_p;
	TH1D* goodtrk_eta;
	TH1D* goodtrk_phi;
	TH1D* goodtrk_dxy;
	TH1D* goodtrk_dz;
	TH1D* goodtrk_d0;
	TH1D* goodtrk_chi2;
	TH1D* goodtrk_validHits;
	TH1D* goodtrk_LostHits;
	TH1D* GoodTracks;
	TH1D* AllTracks;
};

#endif
