#ifndef TreeRead_H
#define TreeRead_H

#include <vector>
#include <iostream>
#include "TLorentzVector.h"
#include "TStopwatch.h"
#include <TTree.h>
#include <TBranch.h>

using namespace std;

//class TTree;
//class TBranch;

class TreeRead {

  public:

    TTree          *fChain;   // pointer to the analyzed TTree or TChain
    Int_t           fCurrent; // current Tree number in a TChain

    // Declaration of leaf types
    Int_t           evNumber;
    Int_t           runNumber;
    Int_t           numberOfTrack;
    vector<Double_t>  *pt;
    vector<Double_t>  *ptError;
    vector<Double_t>  *p;
    vector<Double_t>  *px;
    vector<Double_t>  *py;
    vector<Double_t>  *pz;
    vector<Double_t>  *eta;
    vector<Double_t>  *etaError;
    vector<Double_t>  *phi;
    vector<Double_t>  *phiError;
    vector<Double_t>  *dxy_PV;
    vector<Double_t>  *dxy_0;
    vector<Double_t>  *dxyError;
    vector<Double_t>  *dz;
    vector<Double_t>  *dzError;
    vector<Double_t>  *d0;
    vector<Double_t>  *d0Error;
    vector<Double_t>  *normalizedChi2;
    vector<Double_t>  *newdEdx1;
    vector<Double_t>  *olddEdx1;
    vector<Double_t>  *chi2;
    vector<Double_t>  *ndof;
    vector<int>     *trk_algo;
    vector<Double_t>  *beamSpot_x;
    vector<Double_t>  *referencePoint_x;
    vector<Double_t>  *innerPosition_x;
    vector<Double_t>  *outerPosition_x;
    vector<Double_t>  *beamSpot_y;
    vector<Double_t>  *referencePoint_y;
    vector<Double_t>  *innerPosition_y;
    vector<Double_t>  *outerPosition_y;
    vector<Double_t>  *beamSpot_z;
    vector<Double_t>  *referencePoint_z;
    vector<Double_t>  *innerPosition_z;
    vector<Double_t>  *outerPosition_z;
    vector<int>     *charge;
    vector<int>     *trk_jetRef;
    vector<int>     *numberOfvalidHits;
    vector<int>     *numberOfLostHits;
    vector<bool>    *highPurity_track;

    Int_t           numberOfJet;
    vector<Double_t>  *jet_px;
    vector<Double_t>  *jet_py;
    vector<Double_t>  *jet_pz;
    vector<Double_t>  *jet_energy;
    vector<Double_t>  *jet_eta;
    vector<Double_t>  *jet_phi;
    vector<Double_t>  *jet_ecfr;
    vector<Double_t>  *jet_enfr;
    vector<Double_t>  *jet_hcfr;
    vector<Double_t>  *jet_hnfr;
    vector<Double_t>  *jet_mass;
    vector<Double_t>  *jet_ene_c3;

    vector<unsigned short> *Genp_status;
    vector<int>  *Genp_Id;
    vector<vector<unsigned short> > *Genp_vMom;
    vector<vector<unsigned short> > *Genp_vDau;
    vector<float>   *Genp_charge;
    vector<Double_t>  *Genp_pt;
    vector<Double_t>  *Genp_eta;
    vector<Double_t>  *Genp_phi;
    vector<Double_t>  *Genp_mass;
    vector<Double_t>  *Genp_vx;
    vector<Double_t>  *Genp_vy;
    vector<Double_t>  *Genp_vz;
    vector<Double_t>  *DeltaR_clst;
    vector<Double_t>  *DeltaPt_clst;
    vector<int>     *Genp_idx_clst;

    // List of branches
    TBranch        *b_evNumber;   //!
    TBranch        *b_runNumber;   //!
    TBranch        *b_numberOfTrack;   //!
    TBranch        *b_pt;   //!
    TBranch        *b_ptError;   //!
    TBranch        *b_p;   //!
    TBranch        *b_px;   //!
    TBranch        *b_py;   //!
    TBranch        *b_pz;   //!
    TBranch        *b_eta;   //!
    TBranch        *b_etaError;   //!
    TBranch        *b_phi;   //!
    TBranch        *b_phiError;   //!
    TBranch        *b_dxy_PV;   //!
    TBranch        *b_dxy_0;   //!
    TBranch        *b_dxyError;   //!
    TBranch        *b_dz;   //!
    TBranch        *b_dzError;   //!
    TBranch        *b_d0;   //!
    TBranch        *b_d0Error;   //!
    TBranch        *b_normalizedChi2;   //!
    TBranch        *b_newdEdx1;   //!
    TBranch        *b_olddEdx1;   //!
    TBranch        *b_chi2;   //!
    TBranch        *b_ndof;   //!
    TBranch        *b_trk_algo;   //!
    TBranch        *b_beamSpot_x;   //!
    TBranch        *b_referencePoint_x;   //!
    TBranch        *b_innerPosition_x;   //!
    TBranch        *b_outerPosition_x;   //!
    TBranch        *b_beamSpot_y;   //!
    TBranch        *b_referencePoint_y;   //!
    TBranch        *b_innerPosition_y;   //!
    TBranch        *b_outerPosition_y;   //!
    TBranch        *b_beamSpot_z;   //!
    TBranch        *b_referencePoint_z;   //!
    TBranch        *b_innerPosition_z;   //!
    TBranch        *b_outerPosition_z;   //!
    TBranch        *b_charge;   //!
    TBranch        *b_trk_jetRef;   //!
    TBranch        *b_numberOfvalidHits;   //!
    TBranch        *b_numberOfLostHits;   //!
    TBranch        *b_highPurity_track;   //!
    TBranch        *b_numberOfJet;   //!
    TBranch        *b_jet_px;   //!
    TBranch        *b_jet_py;   //!
    TBranch        *b_jet_pz;   //!
    TBranch        *b_jet_energy;   //!
    TBranch        *b_jet_eta;   //!
    TBranch        *b_jet_phi;   //!
    TBranch        *b_jet_ecfr;   //!
    TBranch        *b_jet_enfr;   //!
    TBranch        *b_jet_hcfr;   //!
    TBranch        *b_jet_hnfr;   //!
    TBranch        *b_jet_mass;   //!
    TBranch        *b_jet_ene_c3;   //!
    TBranch        *b_Genp_status;   //!
    TBranch        *b_Genp_Id;   //!
    TBranch        *b_Genp_vMom;   //!
    TBranch        *b_Genp_vDau;   //!
    TBranch        *b_Genp_charge;   //!
    TBranch        *b_Genp_pt;   //!
    TBranch        *b_Genp_eta;   //!
    TBranch        *b_Genp_phi;   //!
    TBranch        *b_Genp_mass;   //!
    TBranch        *b_Genp_vx;   //!
    TBranch        *b_Genp_vy;   //!
    TBranch        *b_Genp_vz;   //!
    TBranch        *b_DeltaR_clst;   //!
    TBranch        *b_DeltaPt_clst;   //!
    TBranch        *b_Genp_idx_clst;   //!

    TreeRead();
    virtual ~TreeRead();

    // store tree pointer and associate variables to branches
    void init( TTree* tree );
    // store branch pointers
    void getBranches();
    // reset all variables
    void reset();
    // read events and analyze them
    virtual void Loop( int evtmax = 9999999, int ifirst = 0 );
	virtual void StopLoop();
    // user analysis - to be overwritten
	virtual void analyze( int ) { return ; } ;
    virtual void begin_analysis() { return ; } ;
    virtual void end_analysis() { return; } ;
    // structure ...
    virtual Int_t    GetEntry(Long64_t entry);
    virtual Long64_t LoadTree(Long64_t entry);
    virtual void Show( Long64_t entry = -1 );
    virtual Bool_t   Notify();
	void ProgressReport( int entry , Double_t& TotalTime, TStopwatch Twatch );
	int Nof();
	void RemoveRun(vector<int> &remrun);
	void RequestNEvents(int value);
	  void GoodRun(vector<int> &goodrunlist);
	private:

    // tree pointer
    //  TTree* currentTree;
    vector<int> fault;
	int requestedevents;
    vector<int> goodrunonly;
    TreeRead( const TreeRead& st );
    TreeRead& operator=( const TreeRead& st );
	bool stop_loop;
	int nof;
protected:
	int goodeventsfound;
};

#endif
