#include "TreeReindex.h"
#include "TTree.h"
#include "TStopwatch.h"
#include "TLorentzVector.h"
#include <iostream>
#include <vector>

using namespace std ;

TreeReindex::TreeReindex() : fChain( 0 ) {
	reset();
}

TreeReindex::~TreeReindex() {}


void TreeReindex::init( TTree* tree ) {
	// Set branch addresses and branch pointers
	if (!tree) return;
	fChain = tree;
	fCurrent = -1;
	//fChain->SetMakeClass(1);
   	fChain->SetBranchAddress("GoodTrackFourVector",&GoodTrackFourVector,&b_GoodTrackFourVector);
	fChain->SetBranchAddress("GoodTrackCharge",&GoodTrackCharge,&b_GoodTrackCharge);
	fChain->SetBranchAddress("evNumber",&EventId,&b_EventId);
	cout << "Loading all tree in memory" << endl;
	fChain->LoadBaskets();
	cout << "All tree loaded" << endl;
	remapper a;
	a.weigth =0;
	a.secweigth =0;
	a.id =0;
	cout << "Creating Map" << endl;
	if (id_map.max_size () < GetEntries()) cout << "Error Too Much entries" << endl;
	cout << "Max number of entry:" << id_map.max_size () << endl;
	for (unsigned int i=0;i < GetEntries(); i++) {
		a.id = i;
		id_map.push_back(a);
	}
	cout << "Map Ready" << endl;
	// cout <<  id_map.size() << endl;
	Notify();
	return;
}


Bool_t TreeReindex::Notify()
{
	// The Notify() function is called when a new file is opened. This
	// can be either for a new TTree in a TChain or when when a new TTree
	// is started when using PROOF. It is normally not necessary to make changes
	// to the generated code, but the routine can be extended by the
	// user if needed. The return value is currently not used.
	return kTRUE;
}

void TreeReindex::getBranches() {
	return;
}


void TreeReindex::reset() 
{  
	// Set object pointer
    EventId = 0;
	GoodTrackFourVector = 0;
	GoodTrackCharge = 0;
	stop_loop = false;
	id_map.erase(id_map.begin(),id_map.end());
}


Int_t TreeReindex::GetEntry(Long64_t entry) {
	// Read contents of entry.
	if (!fChain) return 0;
	return fChain->GetEntry(entry);
}


Int_t TreeReindex::GetIndexedEntry(Long64_t entry) {
	// Read contents of entry.
	if (!fChain) return 0;
	//if (entry > fChain->GetEntries()) return 0;
	return fChain->GetEntry(id_map[entry].id);
}

void TreeReindex::Show(Long64_t entry) {
	// Print contents of entry.
	// If entry is not specified, print current entry
	if (!fChain) return;
	fChain->Show(entry);
}

Long64_t TreeReindex::GetEntries() {
	// Print contents of entry.
	// If entry is not specified, print current entry
	if (!fChain) return 0;
	return fChain->GetEntries();
}

void TreeReindex::Loop( int evtmax, int ifirst ) {
}

void TreeReindex::StopLoop(){
	stop_loop = true;
}

void TreeReindex::ProgressReport( int entry , Double_t& TotalTime, TStopwatch Twatch ) 
{
	static int j=100;
	if ((entry%j)==0) 
	{
		Double_t Time = Twatch.CpuTime() ;
		TotalTime += Time ;
		if ((entry/j)==9) j*=10;
		cout << " Num " << entry << "  Run:Event analyzed " << ":" << EventId 
		<< " Elapsed Time " << Twatch.CpuTime() << " Total " << TotalTime << endl;
		Twatch.Start() ;
	}  
}

void TreeReindex::SortFileOutput(string insortfileoutput){
	sortfileoutput = insortfileoutput;
}
