#ifndef TreeReindex_H
#define TreeReindex_H

#include <vector>
#include <iostream>
#include <TLorentzVector.h>
#include <TStopwatch.h>
#include <TTree.h>
#include <TBranch.h>
#include "remapper.c"

// Classe VIRTUALE che si occupa della lettura del file che contiene le tracce
// non e' necessario fare alcuna modifica a questa classe per cambiare il
// metodo di combinazione usare delle classe che eriditano da Event Mixer.
// La classe non è utilizzabile da sola e' necessario infatti
// avere un metodo che da il peso e di combinazione che non devono essere definite qui


using namespace std;

class TreeReindex {
	
public:
	
	
    TreeReindex();
    virtual ~TreeReindex();
	
    // store tree pointer and associate variables to branches
    void init( TTree* tree );
    // store branch pointers
    void getBranches();
    // reset all variables
    void reset();
    // read events and analyze them
    virtual void Loop( int evtmax = 9999999, int ifirst = 0 );
	virtual void StopLoop();
    // user analysis - to be overwritten
    virtual void analyze( int ) { return ; } ;
    virtual void begin_analysis() { return ; } ;
    virtual void end_analysis() { return; } ;
	virtual void EvtSort(int maxentry=0)=0;
	virtual void ProgressReport( int entry , Double_t& TotalTime, TStopwatch Twatch );
    // structure ...
    virtual Int_t    GetEntry(Long64_t entry);
    virtual void Show( Long64_t entry = -1 );
    virtual Bool_t   Notify();
	virtual Int_t GetIndexedEntry(Long64_t entry);
	Long64_t GetEntries();
	virtual void SortFileOutput(string insortfileoutput = "SortList.txt") ; // Definisce una sort list di appoggio
	
private:
	
    // tree pointer
    //  TTree* currentTree;
	
    TreeReindex( const TreeReindex& st );
    TreeReindex& operator=( const TreeReindex& st );
	
protected:
	
	TTree          *fChain;   // pointer to the analyzed TTree or TChain
    Int_t           fCurrent; // current Tree number in a TChain
	// Declaration of leaf types
    Int_t           EventId;
	vector<TLorentzVector>* GoodTrackFourVector;
	vector<int>* GoodTrackCharge;
    // List of branches
    TBranch        *b_GoodTrackFourVector;   //!
    TBranch        *b_GoodTrackCharge;   //!
    TBranch        *b_EventId;   //!
	vector<remapper> id_map; // Oggetto contenitore per la mappa degli eventi
	bool stop_loop;
	string sortfileoutput;

};

#endif
