#include "ZrotMix.h"


void PrintMatrix(TMatrixD &M){
	for (int k=0; k<3; k++) {
		for (int l=0; l<3; l++) {
			cout << Form("M[%d][%d] =",l,k) << M[l][k] << ";" << endl;
		}
	}
	cout << endl;
}


void getsferandeta(Double_t value,Double_t &sfer,Double_t &eta){
	sfer = (int)(value/10000.0);
	//cout << sfer << endl;
	eta = value - sfer*10000.0;
	sfer = TMath::Abs(((Double_t)sfer)/10000.0);
	//cout << sfer << endl;
};


ZrotMix::ZrotMix()
{
	matrixlist = new vector<matrixcontainer>; // Indice della classe
	indexed = false; // Parametro di controllo dell'indice
	rotationmatrix = new TRotation(); // Matrice di rotazione
	lastid = -1; // Id di partenza
	deltas = new TH1D("zmixdeltas","zmixdeltas",200,-0.2,0.2);
	deltaeta = new TH1D("zmixdeltaeta","zmixdeltaeta",200,-1.2,0.2);
	deltan = new TH1D("zmixNminusN","zmixNminusN",200,-100.,100.);
}


// =======================================================================
Double_t ZrotMix::ComputeEventWeight()
{
	Double_t SumP2 =0 ; 
	Double_t SphT[]={0,0,0,0,0,0,0,0,0} ; // Inizializzazione di un vettore vuoto per la matrice di sfericita'
	matrixcontainer a;
	//for (int k=0;k<9;k++) a.matrix[k] =0; // Azzero il vettore che uso per salvare gli autovettori
	
	// Calcolo delle entrate matrice di sfericita
	for( unsigned int i=0 ; i< GoodTrackFourVector->size() ; i++ )
    {
		SphT[0] += GoodTrackFourVector->at(i).Px()*GoodTrackFourVector->at(i).Px() ;
		SphT[1] += GoodTrackFourVector->at(i).Px()*GoodTrackFourVector->at(i).Py() ;
		SphT[2] += GoodTrackFourVector->at(i).Px()*GoodTrackFourVector->at(i).Pz() ;
		
		SphT[3] += GoodTrackFourVector->at(i).Py()*GoodTrackFourVector->at(i).Px() ;
		SphT[4] += GoodTrackFourVector->at(i).Py()*GoodTrackFourVector->at(i).Py() ; 
		SphT[5] += GoodTrackFourVector->at(i).Py()*GoodTrackFourVector->at(i).Pz() ; 
		
		SphT[6] += GoodTrackFourVector->at(i).Pz()*GoodTrackFourVector->at(i).Px() ;
		SphT[7] += GoodTrackFourVector->at(i).Pz()*GoodTrackFourVector->at(i).Py() ; 
		SphT[8] += GoodTrackFourVector->at(i).Pz()*GoodTrackFourVector->at(i).Pz() ;
		
		SumP2 += GoodTrackFourVector->at(i).P() * GoodTrackFourVector->at(i).P() ;
    }
	
	if( SumP2 == 0 ) { cout << "ERROR ERROR ERRROR" << endl; return -1 ;}
	
	TMatrixDSym SphM( 3, SphT ) ; // Creazione della matrice
	
	SphM *= 1./SumP2 ;
	TMatrixDSymEigen SphE( SphM ) ;
	TVectorD SphEigenValues = SphE.GetEigenValues() ;
	a.thematrix.ResizeTo(3,3);
	a.thematrix = SphE.GetEigenVectors();
	
	//PrintMatrix(a.thematrix);
	TVector3 vectorforeta (a.thematrix[0][0],a.thematrix[0][1],a.thematrix[0][2]); // Preparo il vettore per calcolare eta
	Double_t sph = 1.5 * (SphEigenValues[1]+SphEigenValues[2]); // Calcolo la sfericita
	a.thematrix*=1./a.thematrix.Determinant() ;
	a.weigth = (100.0*((int)(10000.0*sph))+TMath::Abs(vectorforeta.Eta()))*(vectorforeta.Eta()/TMath::Abs(vectorforeta.Eta())); // Impongo come peso per l'ordinamento interno per usarla dopo
	matrixlist->push_back(a); // Inserisco nell'indice di questa classe
	return a.weigth; // Ritorno il peso per usarlo nell'event Mixer (per il sorting)
}

TMatrixD ZrotMix::GetMatrixFromId(int evid){
	if (!indexed){ indexed=true; sort(matrixlist->begin(),matrixlist->end());} // Nel caso nn sia ordinato la ordino
	return matrixlist->at(evid).thematrix; // ritorno l'array
}

Double_t  ZrotMix::GetWeightFromId(int evid){
	if (!indexed){ indexed=true; sort(matrixlist->begin(),matrixlist->end());} // Nel caso nn sia ordinato la ordino
	return matrixlist->at(evid).weigth; // ritorno il peso
}

ZrotMix::~ZrotMix(){
	deltas->Write();
	deltaeta->Write();
	deltan->Write();
	
	delete deltas;
	delete deltaeta;
	delete deltan;
	delete matrixlist;
	delete rotationmatrix;
}

void ZrotMix::multianalisis( const TLorentzVector &p1, const TLorentzVector &p2,int &ChargeP1,int &ChargeP2, int SizeEv1 ,int SizeEv2 ,int evid){
	TLorentzVector newvector = p2; // Vettore di supporto
	if (evid!=lastid) {
		TMatrixD rota =(GetMatrixFromId(evid+1).Invert())*GetMatrixFromId(evid);
		//rota*=rota.Determinant() ;
		// preparo i nuovi assi come differenza degli autovettori di sfericta dei due eventi
		TVector3 newX (rota[0][0],rota[0][1],rota[0][2]);
		TVector3 newY (rota[1][0],rota[1][1],rota[1][2]);
		TVector3 newZ (rota[2][0],rota[2][1],rota[2][2]);
		delete rotationmatrix;
		rotationmatrix = new TRotation();
		rotationmatrix->RotateAxes(newX,newY,newZ); // Creo la matrice di rotazione.
		lastid=evid;
	}
	newvector.Transform(*rotationmatrix); // Ruoto la traccia	
	Double_t q = GetQ(p1,newvector); // Calcolo la correlazione
	Double_t sfer1,sfer2;
	Double_t eta1,eta2;
	getsferandeta(GetWeightFromId(evid),sfer1,eta1);
	getsferandeta(GetWeightFromId(evid+1),sfer2,eta2);
	
	deltas->Fill(sfer1-sfer2);
	deltaeta->Fill(eta1-eta2);
	deltan->Fill(SizeEv1-SizeEv2);
	
	hmix->Fill(q) ; // riempio l'istogramma
	if (ChargeP1*ChargeP2 > 0){ 
		hmixsc->Fill(q) ;
		hmixschVsN->Fill(q,SizeEv1);
	}else{
		hmixoc->Fill(q) ;
		hmixdchVsN->Fill(q,SizeEv1);
	}
	
	
};


