#include <TH1F.h>
#include <TLorentzVector.h>
#include <TTree.h>
#include <TFile.h>
#include <iostream>
#include <vector>
#include <sstream>
#include <math.h>
#include <string>
#include "myHist.h"
#include "myHist.c"

using namespace std;

int gengraphs(string ifilename,string ofilename);

char* genname(string titolo,int numero){
	stringstream ss;//create a stringstream
	ss << titolo;
	ss << numero;
	return (char*) ss.str().c_str();
}
void help(){
    cout << "Use:" << endl;
    cout << "-i input root file" << endl;
    cout << "-o output root file" << endl;
    cout << "-v verbose" << endl;
	cout << "-h help" << endl;
}

int main( int argc, char* argv[] ) {
	string ifilename,ofilename;
	std::stringstream nomegrafico;
	bool verbose;
	for (int i = 1; i < argc; i++) {
		if (argv[i][0] == '-') {
			switch (argv[i][1]) {
				case 'i':	ifilename = argv[++i];
					break;
				case 'o':	ofilename = argv[++i];
					break;
				case 'v':	verbose = true;
					break;
				case 'h':
					help();
					return 1;
					break;
			}
		}
	}
	if (ifilename.size() < 5){
		cout << "No input file" << endl;
		return 1;
	}
	if (ofilename.size() < 5){
		cout << "No output file" << endl;
		return 1;
	}
	gengraphs(ifilename,ofilename);
	return 0;
}

int gengraphs(string ifilename,string ofilename){
	// Define branch variable:
	vector<TLorentzVector>* GoodTrackFourVector =0;
	vector<int>* GoodTrackCharge=0;
	int EventId;
	char* Pt_plus; char* Eta_plus; char* Phi_plus ; char* P_plus;
	char* Pt_minus; char* Eta_minus; char*Phi_minus; char*P_minus;
	//int n=0;
	// Access to file
	TFile* ifile = new TFile( ifilename.c_str() );
	// Reading tree from file
	TTree *InputTree = dynamic_cast<TTree*>( ifile->Get( "SelectedTracks" ) );
	// Preparing branch access
	InputTree->SetBranchAddress("GoodTrackFourVector",&GoodTrackFourVector);
	InputTree->SetBranchAddress("GoodTrackCharge",&GoodTrackCharge);
	InputTree->SetBranchAddress("OldEventId",&EventId);
	for (unsigned int i =0; i<InputTree->GetEntries(); i++) {
		InputTree->GetEntry(i);
			//cout << ++n << " " << GoodTrackFourVector->size();
		Pt_plus = genname("Pt_plus_",i);
		Eta_plus = genname("Eta_plus_",i) ;
		Phi_plus = genname("Phi_plus_",i);
		P_plus = genname("P_plus_",i) ;
		Pt_minus = genname("Pt_minus",i) ;
		Eta_minus = genname("Eta_minus",i);
		Phi_minus = genname("Phi_minus",i);
		P_minus = genname("P_minus",i) ;
		for (unsigned int k=0; k<GoodTrackFourVector->size(); k++) {
			cout << " " << GoodTrackFourVector->at(k).Pt();
			if (GoodTrackCharge->at(k) > 0){
				// Pt plus
				Hfill( (char*)"Pt_plus" , GoodTrackFourVector->at(k).Pt(), 420, -1,20,1 ) ;
				//Hfill( Pt_plus , GoodTrackFourVector->at(k).Pt(), 420, -1,20,1 ) ;
				// Eta plus
				Hfill( (char*)"Eta_plus" , GoodTrackFourVector->at(k).Eta(),80, -2,2,1 ) ;
				//Hfill( Eta_plus, GoodTrackFourVector->at(k).Eta(),80, -2,2,1 ) ;
				// Phi plus
				Hfill( (char*)"Phi_plus" , GoodTrackFourVector->at(k).Phi(), 160, -4,4,1 ) ;
				//Hfill( Phi_plus , GoodTrackFourVector->at(k).Phi(), 160, -4,4,1 ) ;
				// P plus
				Hfill( (char*)"P_plus" , GoodTrackFourVector->at(k).P(), 500, 0,25,1) ;
				//Hfill( P_plus , GoodTrackFourVector->at(k).P(), 500, 0,25,1) ;
			} else {
				// Pt minus
				Hfill( (char*)"Pt_minus" , GoodTrackFourVector->at(k).Pt(), 420, -1,20,1 ) ;
				//Hfill( Pt_minus, GoodTrackFourVector->at(k).Pt(), 420, -1,20,1 ) ;
				// Eta minus
				Hfill( (char*)"Eta_minus" , GoodTrackFourVector->at(k).Eta(),80, -2,2,1 ) ;
				//Hfill( Eta_minus , GoodTrackFourVector->at(k).Eta(),80, -2,2,1 ) ;
				// Phi minus
				Hfill( (char*)"Phi_minus" , GoodTrackFourVector->at(k).Phi(), 160, -4,4,1 ) ;
				//Hfill(Phi_minus  , GoodTrackFourVector->at(k).Phi(), 160, -4,4,1 ) ;
				// P minus
				Hfill( (char*)"P_minus" , GoodTrackFourVector->at(k).P(), 500, 0,25,1) ;
				//Hfill( P_minus, GoodTrackFourVector->at(k).P(), 500, 0,25,1) ;
			}

		}
		cout << endl;
	}
	SaveHist((char*) ofilename.c_str() ) ;
	return 0;
}
