#include "MassMix.h"
#include "RandomMix.h"
#include "EtaMix.h"
#include "MoltMix.h"
#include "EtaSegMix.h"
#include "EtaSegRotatedMix.h"
#include "SphereMix.h"
#include "ZrotMix.h"
#include <TTree.h>
#include <TBenchmark.h>
#include <signal.h>
#include <TFile.h>
#include <iostream>
#include <stdlib.h>

using namespace std;

// Define Global variable

void help(){
    cout << "Use:" << endl;
    cout << "-i input root file" << endl;
    cout << "-o output root file" << endl;
	cout << "-d output dict file" << endl;
	cout << "-n max number of events" << endl;
	cout << "-h show this help\n" << endl;
	cout << "mix options:" << endl;
	cout << "-2 make only mixing with all events" << endl;
	cout << "-1 make only mixing each event" << endl;
	cout << "without options calculate both" << endl;
	cout << "Sorting methods:" << endl;
	cout << "-m sort using mass" << endl;
	cout << "-e sort using eta" << endl;
	cout << "-y sort using eta segmented" << endl;
	cout << "-k sort using eta segmented and rotated" << endl;
	cout << "-r sort using access sequential (random)" << endl;
	cout << "-s sort using sphericity" << endl;
	cout << "-z sort using sphericity,eta and z-rotated" << endl;
	cout << "\nDefault is mass" << endl;
}

EventMixer *d;

int info_action( int a, void(*b)(int) );
void handle_signal( int code );


int main( int argc, char* argv[] ) {
	string ifilename,ofilename,sfilename;
	int mixtype=0;
	int mix = 3;
	int cycle =0;
	// set file name
	if (argc < 2) {help(); return 1;}
	for (int i = 1; i < argc; i++) {
		if (argv[i][0] == '-') {
			switch (argv[i][1]) {
				case 'o':	ofilename = argv[++i];
					break;
				case 'i':	ifilename = argv[++i];
					break;
				case 'd':	sfilename = argv[++i];
					break;
				case 'm':	mix=3;
					break;
				case 's':	mix=2;
					break;
				case 'e':	mix=1;
					break;
				case 'r':	mix=0;
					break;
				case 'y':	mix=4;
					break;
				case 'z':	mix=5;
					break;
				case 'k':	mix=6;
					break;
				case 'n':	cycle=atoi(argv[++i]);
					break;
				case 'a':	mix=7;
					break;
				case '1':	mixtype=1;
					break;
				case '2':	mixtype=2;
					break;
				case 'h':	help();
					break;
			}
		}
	}
	if (ifilename.size() < 5) {
		help();
		return 1;
	}
	if (ifilename.compare(ifilename.size()-4,4,"root")){
		cout << "Waiting for .root input file" << endl;
		return 1;
	}
	if (ofilename.size() < 5) {
		help();
		return 1;
	}
	if (ofilename.compare(ofilename.size()-4,4,"root")){
		cout << "Waiting for .root output file" << endl;
		return 1;
	}
	//	if (sfilename.size() < 4) { // Non e' necessario un sort file.
	//		help(); 
	//		return 1;
	//	}
	
	switch (mix) {
		case 0:
			d = new RandomMix();
			break;
		case 1:
			d = new EtaMix();
			break;
		case 2:
			d = new SphereMix();
			break;
		case 3:
			d = new MassMix();
			break;
		case 4:
			d = new EtaSegMix();
			break;
		case 5:
			d = new ZrotMix();
			break;
		case 6:
			d = new EtaSegRotatedMix();
		case 7:
			d = new MoltMix();
			break;
	}
	
	info_action( SIGQUIT, handle_signal );
    info_action( SIGINT, handle_signal );
    info_action( SIGUSR1, handle_signal );
	TFile* ifile = new TFile( ifilename.c_str() );
	// Reading tree from file
	TTree *InputTree = dynamic_cast<TTree*>( ifile->Get( "SelectedTracks" ) );
	TBenchmark *gBenchmark = new TBenchmark();
	
	gBenchmark->Start("tcl");
	d->init( InputTree );
	cout << "start sort" << endl;
	if (sfilename.size() > 4) d->SortFileOutput((char*)sfilename.c_str() );
	d->SaveToFile((char*)ofilename.c_str() ) ;
	if (cycle>0){ 
		d->EvtSort(cycle);
	}else{
		d->EvtSort();
	}	
	if (mixtype == 1)d->DisableMixCalculator();
	if (mixtype == 2)d->DisableSignalCalculator();
	
	cout << "End sort" << endl;
	// cout << cycle << endl;
	
	d->Loop();
	cout << Form("%d",d->combination) << endl;
	gBenchmark->Show("tcl");
	delete d;
}

int info_action( int a, void(*b)(int) )
{
    struct sigaction action;
	
    sigemptyset( &action.sa_mask );
    action.sa_flags = 0;
#ifdef SA_RESTART
    action.sa_flags |= SA_RESTART;
#endif
    if (b) {
		action.sa_handler = b;
    } else {
		action.sa_handler = SIG_IGN;
    }
    sigaction( a, &action, NULL );
	
    return 1;
}

void handle_signal( int code )
{
	if (code == SIGUSR1 || code ==SIGINT){
		cout << "Ctrl+c -> Closing" << endl;
		d->StopLoop();
	}
}

