#include "TrackSelector.h"
#include "TChain.h"
#include <TBenchmark.h>
#include <signal.h>
#include <iostream>
#include <fstream>
#include <string.h>
#include <sstream>
#include <stdlib.h>

using namespace std;

// Define Global variable

TrackSelector d;


int info_action( int a, void(*b)(int) );
void handle_signal( int code );

void help(){
    cout << "Use:" << endl;
    cout << "-i input list file" << endl;
    cout << "-o output root file tree" << endl;
    cout << "-t template name" << endl;
    cout << "-h show this help" << endl;
	cout << "-n max of good event stored [Optional]" << endl;
}

int main( int argc, char* argv[] ) {
	string ifilename,ofilename,templatename;
	// set file name
	int max=0;
	if (argc < 2) {help(); return 1;}
	for (int i = 1; i < argc; i++) {
		if (argv[i][0] == '-') {
			switch (argv[i][1]) {
				case 'o':	ofilename = argv[++i];
					break;
				case 'i':	ifilename = argv[++i];
					break;
				case 't':	templatename = argv[++i];
					break;
				case 'n':	max=atoi(argv[++i]);
					break;
				case 'h':	help();
					break;
			}
		}
	}
	
	if (ifilename.size() < 4) {
		cout << "Waiting for input list file" << endl;
		help();
		return 1;
	}
	if (ofilename.size() < 5) {
		help();
		return 1;
	}
	if (templatename.size() < 1) {
                help();
                return 1;
        }
	if (ofilename.compare(ofilename.size()-4,4,"root")){
		cout << "Waiting for .root output file" << endl;
		return 1;
	}
	info_action( SIGQUIT, handle_signal );
    info_action( SIGINT, handle_signal );
    info_action( SIGUSR1, handle_signal );
	TChain *track_chain = new TChain(templatename.c_str());
	TBenchmark *gBenchmark = new TBenchmark();
	ifstream inputlist;
	vector<int> fault_run;
	vector<int> good_run;
	inputlist.open(ifilename.c_str(), ifstream::in);
	char bufferstring[512];
	char temp;
	int k =0;
	cout << "Adding file to chain:" << endl;
	while (inputlist.good()) {
		k++;
		inputlist.getline(bufferstring,500);
		if (strlen(bufferstring)>0){
			if (bufferstring[0] == '#'){ 
				cout << bufferstring << endl;
				continue;
			}
			if (bufferstring[0] == '%'){
				int p;
				int n;
				char *ptr = bufferstring;
				sscanf(ptr, "%c%n", &temp,&n);
				cout << "Run saltati" << flush;
				while (sscanf(ptr, "%d%n", &p,&n) != EOF )
				{
					cout << " " << p << flush;
					fault_run.push_back(p);
					ptr+=n;
				}
				cout << endl;
				continue;
			}
			if (bufferstring[0] == '='){
				int p;
				int n;
				char *ptr = bufferstring;
				sscanf(ptr, "%c%n", &temp,&n);
				cout << "Run considerati" << flush;
				while (sscanf(ptr, "%d%n", &p,&n) != EOF )
				{
					cout << " " << p << flush;
					good_run.push_back(p);
					ptr+=n;
				}
				cout << endl;
				continue;
			}
			track_chain->Add(bufferstring);
			cout << "File " <<  k << ": " << bufferstring << endl;
		}
	}
	inputlist.close();
	
	d.InitOutput(ofilename.c_str());
	if (fault_run.size() > 0) d.RemoveRun(fault_run);
	if (good_run.size() > 0) d.GoodRun(good_run);
	gBenchmark->Start("tcl");
	d.init( track_chain );
	if (max !=0) d.RequestNEvents(max);
	d.Loop();
	cout << Form("%f",d.coutcomb) << endl;
	//if(gfilename.size() > 5) {cout << "forse scrivo pure" << endl; d.SaveToFile((char*) gfilename.c_str()) ; cout << "scrivo pure" << endl;}
	cout << "Accepted tracks: "<< d.GetCounterTotal()<< endl;
	cout << "Good events: "<< d.GetGoodEvent() << endl;
	gBenchmark->Show("tcl");
}

int info_action( int a, void(*b)(int) )
{
    struct sigaction action;
	
    sigemptyset( &action.sa_mask );
    action.sa_flags = 0;
#ifdef SA_RESTART
    action.sa_flags |= SA_RESTART;
#endif
    if (b) {
		action.sa_handler = b;
    } else {
		action.sa_handler = SIG_IGN;
    }
    sigaction( a, &action, NULL );
	
    return 1;
}

void handle_signal( int code )
{
	if (code == SIGUSR1 || code ==SIGINT){
		cout << "Ctrl+c -> Closing" << endl;
		d.StopLoop();
	}
}
